@@ -3,26 +3,27 @@ import base64
 import datetime
 import io
 import json
-import logging
 from typing import Union
 
 import magic
 import requests
-import urllib3
-from pythonjsonlogger import jsonlogger
 
-from pycti.api import LOGGER
+from pycti import __version__
 from pycti.api.opencti_api_connector import OpenCTIApiConnector
+from pycti.api.opencti_api_playbook import OpenCTIApiPlaybook
 from pycti.api.opencti_api_work import OpenCTIApiWork
 from pycti.entities.opencti_attack_pattern import AttackPattern
 from pycti.entities.opencti_campaign import Campaign
-from pycti.entities.opencti_case import Case
+from pycti.entities.opencti_case_incident import CaseIncident
+from pycti.entities.opencti_case_rfi import CaseRfi
+from pycti.entities.opencti_case_rft import CaseRft
 from pycti.entities.opencti_channel import Channel
 from pycti.entities.opencti_course_of_action import CourseOfAction
 from pycti.entities.opencti_data_component import DataComponent
 from pycti.entities.opencti_data_source import DataSource
 from pycti.entities.opencti_event import Event
 from pycti.entities.opencti_external_reference import ExternalReference
+from pycti.entities.opencti_feedback import Feedback
 from pycti.entities.opencti_grouping import Grouping
 from pycti.entities.opencti_identity import Identity
 from pycti.entities.opencti_incident import Incident
@@ -34,6 +35,7 @@ from pycti.entities.opencti_label import Label
 from pycti.entities.opencti_language import Language
 from pycti.entities.opencti_location import Location
 from pycti.entities.opencti_malware import Malware
+from pycti.entities.opencti_malware_analysis import MalwareAnalysis
 from pycti.entities.opencti_marking_definition import MarkingDefinition
 from pycti.entities.opencti_narrative import Narrative
 from pycti.entities.opencti_note import Note
@@ -44,36 +46,25 @@ from pycti.entities.opencti_stix import Stix
 from pycti.entities.opencti_stix_core_object import StixCoreObject
 from pycti.entities.opencti_stix_core_relationship import StixCoreRelationship
 from pycti.entities.opencti_stix_cyber_observable import StixCyberObservable
-from pycti.entities.opencti_stix_cyber_observable_relationship import (
-    StixCyberObservableRelationship,
-)
 from pycti.entities.opencti_stix_domain_object import StixDomainObject
+from pycti.entities.opencti_stix_nested_ref_relationship import (
+    StixNestedRefRelationship,
+)
 from pycti.entities.opencti_stix_object_or_stix_relationship import (
     StixObjectOrStixRelationship,
 )
 from pycti.entities.opencti_stix_sighting_relationship import StixSightingRelationship
+from pycti.entities.opencti_task import Task
 from pycti.entities.opencti_threat_actor import ThreatActor
+from pycti.entities.opencti_threat_actor_group import ThreatActorGroup
+from pycti.entities.opencti_threat_actor_individual import ThreatActorIndividual
 from pycti.entities.opencti_tool import Tool
 from pycti.entities.opencti_vocabulary import Vocabulary
 from pycti.entities.opencti_vulnerability import Vulnerability
+from pycti.utils.opencti_logger import logger
 from pycti.utils.opencti_stix2 import OpenCTIStix2
 from pycti.utils.opencti_stix2_utils import OpenCTIStix2Utils
 
-urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
-
-
-class CustomJsonFormatter(jsonlogger.JsonFormatter):
-    def add_fields(self, log_record, record, message_dict):
-        super(CustomJsonFormatter, self).add_fields(log_record, record, message_dict)
-        if not log_record.get("timestamp"):
-            # This doesn't use record.created, so it is slightly off
-            now = datetime.datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%S.%fZ")
-            log_record["timestamp"] = now
-        if log_record.get("level"):
-            log_record["level"] = log_record["level"].upper()
-        else:
-            log_record["level"] = record.levelname
-
 
 class File:
     def __init__(self, name, data, mime="text/plain"):
@@ -91,7 +82,7 @@ class OpenCTIApiClient:
     :type token: str
     :param log_level: log level for the client
     :type log_level: str, optional
-    :param ssl_verify:
+    :param ssl_verify: Requiring the requests to verify the TLS certificate at the server.
     :type ssl_verify: bool, optional
     :param proxies:
     :type proxies: dict, optional, The proxy configuration, would have `http` and `https` attributes. Defaults to {}
@@ -103,6 +94,10 @@ class OpenCTIApiClient:
         ```
     :param json_logging: format the logs as json if set to True
     :type json_logging: bool, optional
+    :param cert: If String, file path to pem file. If Tuple, a ('path_to_cert.crt', 'path_to_key.key') pair representing the certificate and the key.
+    :type cert: str, tuple, optional
+    :param auth: Add a AuthBase class with custom authentication for you OpenCTI infrastructure.
+    :type auth: requests.auth.AuthBase, optional
     """
 
     def __init__(
@@ -113,11 +108,17 @@ class OpenCTIApiClient:
         ssl_verify=False,
         proxies=None,
         json_logging=False,
+        bundle_send_to_queue=True,
+        cert=None,
+        auth=None,
+        perform_health_check=True,
     ):
         """Constructor method"""
 
         # Check configuration
+        self.bundle_send_to_queue = bundle_send_to_queue
         self.ssl_verify = ssl_verify
+        self.cert = cert
         self.proxies = proxies
         if url is None or len(url) == 0:
             raise ValueError("An URL must be set")
@@ -125,28 +126,26 @@ class OpenCTIApiClient:
             raise ValueError("A TOKEN must be set")
 
         # Configure logger
-        log_level = log_level.upper()
-        LOGGER.setLevel(log_level)
-
-        if json_logging:
-            log_handler = logging.StreamHandler()
-            log_handler.setLevel(log_level)
-            formatter = CustomJsonFormatter(
-                "%(timestamp)s %(level)s %(name)s %(message)s"
-            )
-            log_handler.setFormatter(formatter)
-            logging.basicConfig(handlers=[log_handler], level=log_level, force=True)
-        else:
-            logging.basicConfig(level=log_level)
+        self.logger_class = logger(log_level.upper(), json_logging)
+        self.app_logger = self.logger_class("api")
 
         # Define API
         self.api_token = token
         self.api_url = url + "/graphql"
-        self.request_headers = {"Authorization": "Bearer " + token}
-        self.session = requests.session()
+        self.request_headers = {
+            "User-Agent": "pycti/" + __version__,
+            "Authorization": "Bearer " + token,
+        }
+
+        if auth is not None:
+            self.session = requests.session()
+            self.session.auth = auth
+        else:
+            self.session = requests.session()
 
         # Define the dependencies
         self.work = OpenCTIApiWork(self)
+        self.playbook = OpenCTIApiPlaybook(self)
         self.connector = OpenCTIApiConnector(self)
         self.stix2 = OpenCTIStix2(self)
 
@@ -165,17 +164,24 @@ class OpenCTIApiClient:
         self.stix_cyber_observable = StixCyberObservable(self, File)
         self.stix_core_relationship = StixCoreRelationship(self)
         self.stix_sighting_relationship = StixSightingRelationship(self)
-        self.stix_cyber_observable_relationship = StixCyberObservableRelationship(self)
+        self.stix_nested_ref_relationship = StixNestedRefRelationship(self)
         self.identity = Identity(self)
         self.event = Event(self)
         self.location = Location(self)
         self.threat_actor = ThreatActor(self)
+        self.threat_actor_group = ThreatActorGroup(self)
+        self.threat_actor_individual = ThreatActorIndividual(self)
         self.intrusion_set = IntrusionSet(self)
         self.infrastructure = Infrastructure(self)
         self.campaign = Campaign(self)
-        self.case = Case(self)
+        self.case_incident = CaseIncident(self)
+        self.feedback = Feedback(self)
+        self.case_rfi = CaseRfi(self)
+        self.case_rft = CaseRft(self)
+        self.task = Task(self)
         self.incident = Incident(self)
         self.malware = Malware(self)
+        self.malware_analysis = MalwareAnalysis(self)
         self.tool = Tool(self)
         self.channel = Channel(self)
         self.narrative = Narrative(self)
@@ -193,7 +199,7 @@ class OpenCTIApiClient:
         self.indicator = Indicator(self)
 
         # Check if openCTI is available
-        if not self.health_check():
+        if perform_health_check and not self.health_check():
             raise ValueError(
                 "OpenCTI API is not reachable. Waiting for OpenCTI API to start or check your configuration..."
             )
@@ -201,12 +207,29 @@ class OpenCTIApiClient:
     def set_applicant_id_header(self, applicant_id):
         self.request_headers["opencti-applicant-id"] = applicant_id
 
+    def set_playbook_id_header(self, playbook_id):
+        self.request_headers["opencti-playbook-id"] = playbook_id
+
+    def set_event_id(self, event_id):
+        self.request_headers["opencti-event-id"] = event_id
+
+    def set_synchronized_upsert_header(self, synchronized):
+        self.request_headers["synchronized-upsert"] = (
+            "true" if synchronized is True else "false"
+        )
+
+    def set_previous_standard_header(self, previous_standard):
+        self.request_headers["previous-standard"] = previous_standard
+
+    def get_request_headers(self):
+        return self.request_headers
+
     def set_retry_number(self, retry_number):
         self.request_headers["opencti-retry-number"] = (
             "" if retry_number is None else str(retry_number)
         )
 
-    def query(self, query, variables={}):
+    def query(self, query, variables=None):
         """submit a query to the OpenCTI GraphQL API
 
         :param query: GraphQL query string
@@ -216,7 +239,7 @@ class OpenCTIApiClient:
         :return: returns the response json content
         :rtype: Any
         """
-
+        variables = variables or {}
         query_var = {}
         files_vars = []
         # Implementation of spec https://github.com/jaydenseric/graphql-multipart-request-spec
@@ -305,6 +328,7 @@ class OpenCTIApiClient:
                 files=multipart_files,
                 headers=self.request_headers,
                 verify=self.ssl_verify,
+                cert=self.cert,
                 proxies=self.proxies,
             )
         # If no
@@ -314,6 +338,7 @@ class OpenCTIApiClient:
                 json={"query": query, "variables": variables},
                 headers=self.request_headers,
                 verify=self.ssl_verify,
+                cert=self.cert,
                 proxies=self.proxies,
             )
         # Build response
@@ -326,20 +351,19 @@ class OpenCTIApiClient:
                     if "name" in main_error
                     else main_error["message"]
                 )
-                if "data" in main_error and "reason" in main_error["data"]:
-                    LOGGER.error(main_error["data"]["reason"])
-                    raise ValueError(
-                        {"name": error_name, "message": main_error["data"]["reason"]}
-                    )
-                else:
-                    LOGGER.error(main_error["message"])
-                    raise ValueError(
-                        {"name": error_name, "message": main_error["message"]}
-                    )
+                error_detail = {
+                    "name": error_name,
+                    "error_message": main_error["message"],
+                }
+                meta_data = main_error["data"] if "data" in main_error else {}
+                # Prevent logging of input as bundle is logged differently
+                if meta_data.get("input") is not None:
+                    del meta_data["input"]
+                value_error = {**error_detail, **meta_data}
+                raise ValueError(value_error)
             else:
                 return result
         else:
-            LOGGER.info(r.text)
             raise ValueError(r.text)
 
     def fetch_opencti_file(self, fetch_uri, binary=False, serialize=False):
@@ -353,7 +377,13 @@ class OpenCTIApiClient:
         :rtype: str or bytes
         """
 
-        r = self.session.get(fetch_uri, headers=self.request_headers)
+        r = self.session.get(
+            fetch_uri,
+            headers=self.request_headers,
+            verify=self.ssl_verify,
+            cert=self.cert,
+            proxies=self.proxies,
+        )
         if binary:
             if serialize:
                 return base64.b64encode(r.content).decode("utf-8")
@@ -362,23 +392,6 @@ class OpenCTIApiClient:
             return base64.b64encode(r.text).decode("utf-8")
         return r.text
 
-    def log(self, level, message):
-        """log a message with defined log level
-        :param level: must be a valid logging log level (debug, info, warning, error)
-        :type level: str
-        :param message: the message to log
-        :type message: str
-        """
-
-        if level == "debug":
-            LOGGER.debug(message)
-        elif level == "info":
-            LOGGER.info(message)
-        elif level == "warning":
-            LOGGER.warning(message)
-        elif level == "error":
-            LOGGER.error(message)
-
     def health_check(self):
         """submit an example request to the OpenCTI API.
 
@@ -386,10 +399,20 @@ class OpenCTIApiClient:
         :rtype: bool
         """
         try:
-            test = self.threat_actor.list(first=1)
+            self.app_logger.info("Health check (platform version)...")
+            test = self.query(
+                """
+                  query {
+                    about {
+                      version
+                    }
+                  }
+                """
+            )
             if test is not None:
                 return True
-        except:
+        except Exception as err:  # pylint: disable=broad-except
+            self.app_logger.error(str(err))
             return False
         return False
 
@@ -400,7 +423,7 @@ class OpenCTIApiClient:
         rtype: dict
         """
 
-        LOGGER.info("Getting logs worker config...")
+        self.app_logger.info("Getting logs worker config...")
         query = """
             query LogsWorkerConfig {
                 logsWorkerConfig {
@@ -467,6 +490,18 @@ class OpenCTIApiClient:
             result = []
         if data is None:
             return result
+
+        # Data can be multiple in edges or directly.
+        # -- When data is directly a listing
+        if isinstance(data, list):
+            for row in data:
+                if with_pagination:
+                    result["entities"].append(self.process_multiple_fields(row))
+                else:
+                    result.append(self.process_multiple_fields(row))
+            return result
+
+        # -- When data is wrapper in edges
         for edge in (
             data["edges"] if "edges" in data and data["edges"] is not None else []
         ):
@@ -475,6 +510,8 @@ class OpenCTIApiClient:
                 result["entities"].append(self.process_multiple_fields(row))
             else:
                 result.append(self.process_multiple_fields(row))
+
+        # -- Add page info if required
         if with_pagination and "pageInfo" in data:
             result["pagination"] = data["pageInfo"]
         return result
@@ -593,18 +630,19 @@ class OpenCTIApiClient:
         """upload a file to OpenCTI API
 
         :param `**kwargs`: arguments for file upload (required: `file_name` and `data`)
-        :return: returns the query respons for the file upload
+        :return: returns the query response for the file upload
         :rtype: dict
         """
 
         file_name = kwargs.get("file_name", None)
+        file_markings = kwargs.get("file_markings", None)
         data = kwargs.get("data", None)
         mime_type = kwargs.get("mime_type", "text/plain")
         if file_name is not None:
-            LOGGER.info("Uploading a file.")
+            self.app_logger.info("Uploading a file.")
             query = """
-                mutation UploadImport($file: Upload!) {
-                    uploadImport(file: $file) {
+                mutation UploadImport($file: Upload!, $fileMarkings: [String]) {
+                    uploadImport(file: $file, fileMarkings: $fileMarkings) {
                         id
                         name
                     }
@@ -616,10 +654,13 @@ class OpenCTIApiClient:
                     mime_type = "application/json"
                 else:
                     mime_type = magic.from_file(file_name, mime=True)
-
-            return self.query(query, {"file": (File(file_name, data, mime_type))})
+            query_vars = {"file": (File(file_name, data, mime_type))}
+            # optional file markings
+            if file_markings is not None:
+                query_vars["fileMarkings"] = file_markings
+            return self.query(query, query_vars)
         else:
-            LOGGER.error("[upload] Missing parameter: file_name")
+            self.app_logger.error("[upload] Missing parameter: file_name")
             return None
 
     def upload_pending_file(self, **kwargs):
@@ -636,7 +677,7 @@ class OpenCTIApiClient:
         entity_id = kwargs.get("entity_id", None)
 
         if file_name is not None:
-            LOGGER.info("Uploading a file.")
+            self.app_logger.info("Uploading a file.")
             query = """
                     mutation UploadPending($file: Upload!, $entityId: String) {
                         uploadPending(file: $file, entityId: $entityId) {
@@ -656,7 +697,7 @@ class OpenCTIApiClient:
                 {"file": (File(file_name, data, mime_type)), "entityId": entity_id},
             )
         else:
-            LOGGER.error("[upload] Missing parameter: file_name")
+            self.app_logger.error("[upload] Missing parameter: file_name")
             return None
 
     def get_stix_content(self, id):
@@ -666,7 +707,7 @@ class OpenCTIApiClient:
         rtype: dict
         """
 
-        LOGGER.info("Entity in JSON %s", id)
+        self.app_logger.info("Entity in JSON", {"id": id})
         query = """
             query StixQuery($id: String!) {
                 stix(id: $id)
@@ -701,6 +742,8 @@ class OpenCTIApiClient:
             return object["extensions"][
                 "extension-definition--f93e2c80-4231-4f9a-af8b-95c9bd566a82"
             ][key]
+        elif key in object and key not in ["type"]:
+            return object[key]
         return None
 
     @staticmethod
