@@ -26,6 +26,21 @@ class ExternalReference:
             url
             hash
             external_id
+        """
+        self.properties_with_files = """
+            id
+            standard_id
+            entity_type
+            parent_types
+            created_at
+            updated_at
+            created
+            modified
+            source_name
+            description
+            url
+            hash
+            external_id
             importFiles {
                 edges {
                     node {
@@ -71,21 +86,25 @@ class ExternalReference:
         custom_attributes = kwargs.get("customAttributes", None)
         get_all = kwargs.get("getAll", False)
         with_pagination = kwargs.get("withPagination", False)
+        with_files = kwargs.get("withFiles", False)
         if get_all:
-            first = 500
+            first = 100
 
-        self.opencti.log(
-            "info",
-            "Listing External-Reference with filters " + json.dumps(filters) + ".",
+        self.opencti.app_logger.info(
+            "Listing External-Reference with filters", {"filters": json.dumps(filters)}
         )
         query = (
             """
-            query ExternalReferences($filters: [ExternalReferencesFiltering], $first: Int, $after: ID, $orderBy: ExternalReferencesOrdering, $orderMode: OrderingMode) {
+            query ExternalReferences($filters: FilterGroup, $first: Int, $after: ID, $orderBy: ExternalReferencesOrdering, $orderMode: OrderingMode) {
                 externalReferences(filters: $filters, first: $first, after: $after, orderBy: $orderBy, orderMode: $orderMode) {
                     edges {
                         node {
                             """
-            + (custom_attributes if custom_attributes is not None else self.properties)
+            + (
+                custom_attributes
+                if custom_attributes is not None
+                else (self.properties_with_files if with_files else self.properties)
+            )
             + """
                         }
                     }
@@ -110,9 +129,34 @@ class ExternalReference:
                 "orderMode": order_mode,
             },
         )
-        return self.opencti.process_multiple(
-            result["data"]["externalReferences"], with_pagination
-        )
+        if get_all:
+            final_data = []
+            data = self.opencti.process_multiple(result["data"]["externalReferences"])
+            final_data = final_data + data
+            while result["data"]["externalReferences"]["pageInfo"]["hasNextPage"]:
+                after = result["data"]["externalReferences"]["pageInfo"]["endCursor"]
+                self.opencti.app_logger.info(
+                    "Listing External-References", {"after": after}
+                )
+                result = self.opencti.query(
+                    query,
+                    {
+                        "filters": filters,
+                        "first": first,
+                        "after": after,
+                        "orderBy": order_by,
+                        "orderMode": order_mode,
+                    },
+                )
+                data = self.opencti.process_multiple(
+                    result["data"]["externalReferences"]
+                )
+                final_data = final_data + data
+            return final_data
+        else:
+            return self.opencti.process_multiple(
+                result["data"]["externalReferences"], with_pagination
+            )
 
     """
         Read a External-Reference object
@@ -126,7 +170,7 @@ class ExternalReference:
         id = kwargs.get("id", None)
         filters = kwargs.get("filters", None)
         if id is not None:
-            self.opencti.log("info", "Reading External-Reference {" + id + "}.")
+            self.opencti.app_logger.info("Reading External-Reference", {"id": id})
             query = (
                 """
                 query ExternalReference($id: String!) {
@@ -149,9 +193,8 @@ class ExternalReference:
             else:
                 return None
         else:
-            self.opencti.log(
-                "error",
-                "[opencti_external_reference] Missing parameters: id or filters",
+            self.opencti.app_logger.error(
+                "[opencti_external_reference] Missing parameters: id or filters"
             )
             return None
 
@@ -173,13 +216,13 @@ class ExternalReference:
         x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
         update = kwargs.get("update", False)
 
-        if source_name is not None and url is not None:
-            self.opencti.log(
-                "info", "Creating External Reference {" + source_name + "}."
+        if source_name is not None or url is not None:
+            self.opencti.app_logger.info(
+                "Creating External Reference", {"source_name": source_name}
             )
             query = (
                 """
-                mutation ExternalReferenceAdd($input: ExternalReferenceAddInput) {
+                mutation ExternalReferenceAdd($input: ExternalReferenceAddInput!) {
                     externalReferenceAdd(input: $input) {
                         """
                 + self.properties
@@ -208,9 +251,8 @@ class ExternalReference:
                 result["data"]["externalReferenceAdd"]
             )
         else:
-            self.opencti.log(
-                "error",
-                "[opencti_external_reference] Missing parameters: source_name and url",
+            self.opencti.app_logger.error(
+                "[opencti_external_reference] Missing parameters: source_name and url"
             )
 
     """
@@ -226,13 +268,15 @@ class ExternalReference:
         id = kwargs.get("id", None)
         file_name = kwargs.get("file_name", None)
         data = kwargs.get("data", None)
+        version = kwargs.get("version", None)
         mime_type = kwargs.get("mime_type", "text/plain")
+        no_trigger_import = kwargs.get("no_trigger_import", False)
         if id is not None and file_name is not None:
             final_file_name = os.path.basename(file_name)
             query = """
-                mutation ExternalReferenceEdit($id: ID!, $file: Upload!) {
+                mutation ExternalReferenceEdit($id: ID!, $file: Upload!, $version: DateTime, $noTriggerImport: Boolean) {
                     externalReferenceEdit(id: $id) {
-                        importPush(file: $file) {
+                        importPush(file: $file, version: $version, noTriggerImport: $noTriggerImport) {
                             id
                             name
                         }
@@ -245,22 +289,26 @@ class ExternalReference:
                     mime_type = "application/json"
                 else:
                     mime_type = magic.from_file(file_name, mime=True)
-            self.opencti.log(
-                "info",
-                "Uploading a file {"
-                + final_file_name
-                + "} in Stix-Domain-Object {"
-                + id
-                + "}.",
+            self.opencti.app_logger.info(
+                "Uploading a file in Stix-Domain-Object",
+                {"file": final_file_name, "id": id},
             )
             return self.opencti.query(
                 query,
-                {"id": id, "file": (self.file(final_file_name, data, mime_type))},
+                {
+                    "id": id,
+                    "file": (self.file(final_file_name, data, mime_type)),
+                    "version": version,
+                    "noTriggerImport": (
+                        no_trigger_import
+                        if isinstance(no_trigger_import, bool)
+                        else no_trigger_import == "True"
+                    ),
+                },
             )
         else:
-            self.opencti.log(
-                "error",
-                "[opencti_stix_domain_object] Missing parameters: id or file_name",
+            self.opencti.app_logger.error(
+                "[opencti_stix_domain_object] Missing parameters: id or file_name"
             )
             return None
 
@@ -276,7 +324,7 @@ class ExternalReference:
         id = kwargs.get("id", None)
         input = kwargs.get("input", None)
         if id is not None and input is not None:
-            self.opencti.log("info", "Updating External-Reference {" + id + "}.")
+            self.opencti.app_logger.info("Updating External-Reference", {"id": id})
             query = """
                     mutation ExternalReferenceEdit($id: ID!, $input: [EditInput]!) {
                         externalReferenceEdit(id: $id) {
@@ -291,14 +339,13 @@ class ExternalReference:
                 result["data"]["externalReferenceEdit"]["fieldPatch"]
             )
         else:
-            self.opencti.log(
-                "error",
-                "[opencti_external_reference] Missing parameters: id and key and value",
+            self.opencti.app_logger.error(
+                "[opencti_external_reference] Missing parameters: id and key and value"
             )
             return None
 
     def delete(self, id):
-        self.opencti.log("info", "Deleting External-Reference " + id + "...")
+        self.opencti.app_logger.info("Deleting External-Reference", {"id": id})
         query = """
              mutation ExternalReferenceEdit($id: ID!) {
                  externalReferenceEdit(id: $id) {
@@ -310,10 +357,7 @@ class ExternalReference:
 
     def list_files(self, **kwargs):
         id = kwargs.get("id", None)
-        self.opencti.log(
-            "info",
-            "Listing files of External-Reference { " + id + " }",
-        )
+        self.opencti.app_logger.info("Listing files of External-Reference", {"id": id})
         query = """
             query externalReference($id: String!) {
                 externalReference(id: $id) {
