@@ -5,7 +5,10 @@ import uuid
 
 from stix2.canonicalization.Canonicalize import canonicalize
 
-from pycti.entities import LOGGER
+from .indicator.opencti_indicator_properties import (
+    INDICATOR_PROPERTIES,
+    INDICATOR_PROPERTIES_WITH_FILES,
+)
 
 
 class Indicator:
@@ -16,154 +19,8 @@ class Indicator:
 
     def __init__(self, opencti):
         self.opencti = opencti
-        self.properties = """
-            id
-            standard_id
-            entity_type
-            parent_types
-            spec_version
-            created_at
-            updated_at
-            createdBy {
-                ... on Identity {
-                    id
-                    standard_id
-                    entity_type
-                    parent_types
-                    spec_version
-                    identity_class
-                    name
-                    description
-                    roles
-                    contact_information
-                    x_opencti_aliases
-                    created
-                    modified
-                    objectLabel {
-                        edges {
-                            node {
-                                id
-                                value
-                                color
-                            }
-                        }
-                    }
-                }
-                ... on Organization {
-                    x_opencti_organization_type
-                    x_opencti_reliability
-                }
-                ... on Individual {
-                    x_opencti_firstname
-                    x_opencti_lastname
-                }
-            }
-            objectMarking {
-                edges {
-                    node {
-                        id
-                        standard_id
-                        entity_type
-                        definition_type
-                        definition
-                        created
-                        modified
-                        x_opencti_order
-                        x_opencti_color
-                    }
-                }
-            }
-            objectLabel {
-                edges {
-                    node {
-                        id
-                        value
-                        color
-                    }
-                }
-            }
-            externalReferences {
-                edges {
-                    node {
-                        id
-                        standard_id
-                        entity_type
-                        source_name
-                        description
-                        url
-                        hash
-                        external_id
-                        created
-                        modified
-                        importFiles {
-                            edges {
-                                node {
-                                    id
-                                    name
-                                    size
-                                    metaData {
-                                        mimetype
-                                        version
-                                    }
-                                }
-                            }
-                        }
-                    }
-                }
-            }
-            revoked
-            confidence
-            created
-            modified
-            pattern_type
-            pattern_version
-            pattern
-            name
-            description
-            indicator_types
-            valid_from
-            valid_until
-            x_opencti_score
-            x_opencti_detection
-            x_opencti_main_observable_type
-            x_mitre_platforms
-            observables {
-                edges {
-                    node {
-                        id
-                        entity_type
-                        observable_value
-                    }
-                }
-            }
-            killChainPhases {
-                edges {
-                    node {
-                        id
-                        standard_id
-                        entity_type
-                        kill_chain_name
-                        phase_name
-                        x_opencti_order
-                        created
-                        modified
-                    }
-                }
-            }
-            importFiles {
-                edges {
-                    node {
-                        id
-                        name
-                        size
-                        metaData {
-                            mimetype
-                            version
-                        }
-                    }
-                }
-            }
-        """
+        self.properties = INDICATOR_PROPERTIES
+        self.properties_with_files = INDICATOR_PROPERTIES_WITH_FILES
 
     @staticmethod
     def generate_id(pattern):
@@ -201,18 +58,25 @@ class Indicator:
         custom_attributes = kwargs.get("customAttributes", None)
         get_all = kwargs.get("getAll", False)
         with_pagination = kwargs.get("withPagination", False)
+        with_files = kwargs.get("withFiles", False)
         if get_all:
             first = 100
 
-        LOGGER.info("Listing Indicators with filters %s.", json.dumps(filters))
+        self.opencti.app_logger.info(
+            "Listing Indicators with filters", {"filters": json.dumps(filters)}
+        )
         query = (
             """
-                query Indicators($filters: [IndicatorsFiltering], $search: String, $first: Int, $after: ID, $orderBy: IndicatorsOrdering, $orderMode: OrderingMode) {
+                query Indicators($filters: FilterGroup, $search: String, $first: Int, $after: ID, $orderBy: IndicatorsOrdering, $orderMode: OrderingMode) {
                     indicators(filters: $filters, search: $search, first: $first, after: $after, orderBy: $orderBy, orderMode: $orderMode) {
                         edges {
                             node {
                                 """
-            + (custom_attributes if custom_attributes is not None else self.properties)
+            + (
+                custom_attributes
+                if custom_attributes is not None
+                else (self.properties_with_files if with_files else self.properties)
+            )
             + """
                         }
                     }
@@ -244,7 +108,7 @@ class Indicator:
             final_data = final_data + data
             while result["data"]["indicators"]["pageInfo"]["hasNextPage"]:
                 after = result["data"]["indicators"]["pageInfo"]["endCursor"]
-                LOGGER.info("Listing Indicators after " + after)
+                self.opencti.app_logger.info("Listing Indicators", {"after": after})
                 result = self.opencti.query(
                     query,
                     {
@@ -274,7 +138,7 @@ class Indicator:
 
         Note: either `id` or `filters` is required.
 
-        :param str id: the id of the Threat-Actor
+        :param str id: the id of the Threat-Actor-Group
         :param list filters: the filters to apply if no id provided
 
         :return: Indicator object
@@ -284,8 +148,9 @@ class Indicator:
         id = kwargs.get("id", None)
         filters = kwargs.get("filters", None)
         custom_attributes = kwargs.get("customAttributes", None)
+        with_files = kwargs.get("withFiles", False)
         if id is not None:
-            LOGGER.info("Reading Indicator {%s}.", id)
+            self.opencti.app_logger.info("Reading Indicator", {"id": id})
             query = (
                 """
                     query Indicator($id: String!) {
@@ -294,7 +159,7 @@ class Indicator:
                 + (
                     custom_attributes
                     if custom_attributes is not None
-                    else self.properties
+                    else (self.properties_with_files if with_files else self.properties)
                 )
                 + """
                     }
@@ -310,7 +175,9 @@ class Indicator:
             else:
                 return None
         else:
-            LOGGER.error("[opencti_indicator] Missing parameters: id or filters")
+            self.opencti.app_logger.error(
+                "[opencti_indicator] Missing parameters: id or filters"
+            )
             return None
 
     def create(self, **kwargs):
@@ -337,7 +204,7 @@ class Indicator:
         pattern_type = kwargs.get("pattern_type", None)
         pattern_version = kwargs.get("pattern_version", None)
         pattern = kwargs.get("pattern", None)
-        name = kwargs.get("name", None)
+        name = kwargs.get("name", kwargs.get("pattern", None))
         description = kwargs.get("description", None)
         indicator_types = kwargs.get("indicator_types", None)
         valid_from = kwargs.get("valid_from", None)
@@ -352,18 +219,20 @@ class Indicator:
         x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
         create_observables = kwargs.get("x_opencti_create_observables", False)
         granted_refs = kwargs.get("objectOrganization", None)
+        x_opencti_workflow_id = kwargs.get("x_opencti_workflow_id", None)
         update = kwargs.get("update", False)
 
         if (
             name is not None
             and pattern is not None
+            and pattern_type is not None
             and x_opencti_main_observable_type is not None
         ):
             if x_opencti_main_observable_type == "File":
                 x_opencti_main_observable_type = "StixFile"
-            LOGGER.info("Creating Indicator {%s}.", name)
+            self.opencti.app_logger.info("Creating Indicator", {"name": name})
             query = """
-                mutation IndicatorAdd($input: IndicatorAddInput) {
+                mutation IndicatorAdd($input: IndicatorAddInput!) {
                     indicatorAdd(input: $input) {
                         id
                         standard_id
@@ -381,8 +250,6 @@ class Indicator:
                     }
                 }
             """
-            if pattern_type is None:
-                pattern_type = "stix2"
             result = self.opencti.query(
                 query,
                 {
@@ -413,15 +280,16 @@ class Indicator:
                         "x_opencti_stix_ids": x_opencti_stix_ids,
                         "killChainPhases": kill_chain_phases,
                         "createObservables": create_observables,
+                        "x_opencti_workflow_id": x_opencti_workflow_id,
                         "update": update,
                     }
                 },
             )
             return self.opencti.process_multiple_fields(result["data"]["indicatorAdd"])
         else:
-            LOGGER.error(
+            self.opencti.app_logger.error(
                 "[opencti_indicator] Missing parameters: "
-                "name or pattern or x_opencti_main_observable_type"
+                "name or pattern or pattern_type or x_opencti_main_observable_type"
             )
 
     def add_stix_cyber_observable(self, **kwargs):
@@ -441,17 +309,16 @@ class Indicator:
             if indicator is None:
                 indicator = self.read(id=id)
             if indicator is None:
-                LOGGER.error(
+                self.opencti.app_logger.error(
                     "[opencti_indicator] Cannot add Object Ref, indicator not found"
                 )
                 return False
             if stix_cyber_observable_id in indicator["observablesIds"]:
                 return True
             else:
-                LOGGER.info(
-                    "Adding Stix-Observable {%s} to Indicator {%s}",
-                    stix_cyber_observable_id,
-                    id,
+                self.opencti.app_logger.info(
+                    "Adding Stix-Observable to Indicator",
+                    {"observable": stix_cyber_observable_id, "indicator": id},
                 )
                 query = """
                     mutation StixCoreRelationshipAdd($input: StixCoreRelationshipAddInput!) {
@@ -473,7 +340,7 @@ class Indicator:
                 )
                 return True
         else:
-            LOGGER.error(
+            self.opencti.app_logger.error(
                 "[opencti_indicator] Missing parameters: id and stix cyber_observable_id"
             )
             return False
@@ -495,13 +362,13 @@ class Indicator:
         if stix_object is not None:
             # Search in extensions
             if "x_opencti_score" not in stix_object:
-                stix_object[
-                    "x_opencti_score"
-                ] = self.opencti.get_attribute_in_extension("score", stix_object)
+                stix_object["x_opencti_score"] = (
+                    self.opencti.get_attribute_in_extension("score", stix_object)
+                )
             if "x_opencti_detection" not in stix_object:
-                stix_object[
-                    "x_opencti_detection"
-                ] = self.opencti.get_attribute_in_extension("detection", stix_object)
+                stix_object["x_opencti_detection"] = (
+                    self.opencti.get_attribute_in_extension("detection", stix_object)
+                )
             if (
                 "x_opencti_main_observable_type" not in stix_object
                 and self.opencti.get_attribute_in_extension(
@@ -509,98 +376,135 @@ class Indicator:
                 )
                 is not None
             ):
-                stix_object[
-                    "x_opencti_main_observable_type"
-                ] = self.opencti.get_attribute_in_extension(
-                    "main_observable_type", stix_object
+                stix_object["x_opencti_main_observable_type"] = (
+                    self.opencti.get_attribute_in_extension(
+                        "main_observable_type", stix_object
+                    )
                 )
             if "x_opencti_create_observables" not in stix_object:
-                stix_object[
-                    "x_opencti_create_observables"
-                ] = self.opencti.get_attribute_in_extension(
-                    "create_observables", stix_object
+                stix_object["x_opencti_create_observables"] = (
+                    self.opencti.get_attribute_in_extension(
+                        "create_observables", stix_object
+                    )
                 )
             if "x_opencti_stix_ids" not in stix_object:
-                stix_object[
-                    "x_opencti_stix_ids"
-                ] = self.opencti.get_attribute_in_extension("stix_ids", stix_object)
-            if "granted_refs" not in stix_object:
-                stix_object["granted_refs"] = self.opencti.get_attribute_in_extension(
-                    "granted_refs", stix_object
+                stix_object["x_opencti_stix_ids"] = (
+                    self.opencti.get_attribute_in_extension("stix_ids", stix_object)
+                )
+            if "x_opencti_granted_refs" not in stix_object:
+                stix_object["x_opencti_granted_refs"] = (
+                    self.opencti.get_attribute_in_extension("granted_refs", stix_object)
+                )
+            if "x_opencti_workflow_id" not in stix_object:
+                stix_object["x_opencti_workflow_id"] = (
+                    self.opencti.get_attribute_in_extension("workflow_id", stix_object)
                 )
 
             return self.create(
                 stix_id=stix_object["id"],
-                createdBy=extras["created_by_id"]
-                if "created_by_id" in extras
-                else None,
-                objectMarking=extras["object_marking_ids"]
-                if "object_marking_ids" in extras
-                else None,
-                objectLabel=extras["object_label_ids"]
-                if "object_label_ids" in extras
-                else [],
-                externalReferences=extras["external_references_ids"]
-                if "external_references_ids" in extras
-                else [],
+                createdBy=(
+                    extras["created_by_id"] if "created_by_id" in extras else None
+                ),
+                objectMarking=(
+                    extras["object_marking_ids"]
+                    if "object_marking_ids" in extras
+                    else None
+                ),
+                objectLabel=(
+                    extras["object_label_ids"] if "object_label_ids" in extras else None
+                ),
+                externalReferences=(
+                    extras["external_references_ids"]
+                    if "external_references_ids" in extras
+                    else None
+                ),
                 revoked=stix_object["revoked"] if "revoked" in stix_object else None,
-                confidence=stix_object["confidence"]
-                if "confidence" in stix_object
-                else None,
+                confidence=(
+                    stix_object["confidence"] if "confidence" in stix_object else None
+                ),
                 lang=stix_object["lang"] if "lang" in stix_object else None,
                 created=stix_object["created"] if "created" in stix_object else None,
                 modified=stix_object["modified"] if "modified" in stix_object else None,
-                pattern_type=stix_object["pattern_type"]
-                if "pattern_type" in stix_object
-                else None,
-                pattern_version=stix_object["pattern_version"]
-                if "pattern_version" in stix_object
-                else None,
+                pattern_type=(
+                    stix_object["pattern_type"]
+                    if "pattern_type" in stix_object
+                    else None
+                ),
+                pattern_version=(
+                    stix_object["pattern_version"]
+                    if "pattern_version" in stix_object
+                    else None
+                ),
                 pattern=stix_object["pattern"] if "pattern" in stix_object else "",
-                name=stix_object["name"]
-                if "name" in stix_object
-                else stix_object["pattern"],
-                description=self.opencti.stix2.convert_markdown(
-                    stix_object["description"]
-                )
-                if "description" in stix_object
-                else "",
-                indicator_types=stix_object["indicator_types"]
-                if "indicator_types" in stix_object
-                else None,
-                valid_from=stix_object["valid_from"]
-                if "valid_from" in stix_object
-                else None,
-                valid_until=stix_object["valid_until"]
-                if "valid_until" in stix_object
-                else None,
-                x_opencti_score=stix_object["x_opencti_score"]
-                if "x_opencti_score" in stix_object
-                else 50,
-                x_opencti_detection=stix_object["x_opencti_detection"]
-                if "x_opencti_detection" in stix_object
-                else False,
-                x_mitre_platforms=stix_object["x_mitre_platforms"]
-                if "x_mitre_platforms" in stix_object
-                else None,
-                x_opencti_main_observable_type=stix_object[
-                    "x_opencti_main_observable_type"
-                ]
-                if "x_opencti_main_observable_type" in stix_object
-                else "Unknown",
-                killChainPhases=extras["kill_chain_phases_ids"]
-                if "kill_chain_phases_ids" in extras
-                else None,
-                x_opencti_stix_ids=stix_object["x_opencti_stix_ids"]
-                if "x_opencti_stix_ids" in stix_object
-                else None,
-                x_opencti_create_observables=stix_object["x_opencti_create_observables"]
-                if "x_opencti_create_observables" in stix_object
-                else False,
-                objectOrganization=stix_object["granted_refs"]
-                if "granted_refs" in stix_object
-                else None,
+                name=(
+                    stix_object["name"]
+                    if "name" in stix_object
+                    else stix_object["pattern"]
+                ),
+                description=(
+                    self.opencti.stix2.convert_markdown(stix_object["description"])
+                    if "description" in stix_object
+                    else None
+                ),
+                indicator_types=(
+                    stix_object["indicator_types"]
+                    if "indicator_types" in stix_object
+                    else None
+                ),
+                valid_from=(
+                    stix_object["valid_from"] if "valid_from" in stix_object else None
+                ),
+                valid_until=(
+                    stix_object["valid_until"] if "valid_until" in stix_object else None
+                ),
+                x_opencti_score=(
+                    stix_object["x_opencti_score"]
+                    if "x_opencti_score" in stix_object
+                    else 50
+                ),
+                x_opencti_detection=(
+                    stix_object["x_opencti_detection"]
+                    if "x_opencti_detection" in stix_object
+                    else False
+                ),
+                x_mitre_platforms=(
+                    stix_object["x_mitre_platforms"]
+                    if "x_mitre_platforms" in stix_object
+                    else None
+                ),
+                x_opencti_main_observable_type=(
+                    stix_object["x_opencti_main_observable_type"]
+                    if "x_opencti_main_observable_type" in stix_object
+                    else "Unknown"
+                ),
+                killChainPhases=(
+                    extras["kill_chain_phases_ids"]
+                    if "kill_chain_phases_ids" in extras
+                    else None
+                ),
+                x_opencti_stix_ids=(
+                    stix_object["x_opencti_stix_ids"]
+                    if "x_opencti_stix_ids" in stix_object
+                    else None
+                ),
+                x_opencti_create_observables=(
+                    stix_object["x_opencti_create_observables"]
+                    if "x_opencti_create_observables" in stix_object
+                    else False
+                ),
+                objectOrganization=(
+                    stix_object["x_opencti_granted_refs"]
+                    if "x_opencti_granted_refs" in stix_object
+                    else None
+                ),
+                x_opencti_workflow_id=(
+                    stix_object["x_opencti_workflow_id"]
+                    if "x_opencti_workflow_id" in stix_object
+                    else None
+                ),
                 update=update,
             )
         else:
-            LOGGER.error("[opencti_attack_pattern] Missing parameters: stixObject")
+            self.opencti.app_logger.error(
+                "[opencti_indicator] Missing parameters: stixObject"
+            )
