@@ -1,6 +1,9 @@
 # coding: utf-8
 
 import json
+import uuid
+
+from stix2.canonicalization.Canonicalize import canonicalize
 
 
 class Location:
@@ -21,6 +24,7 @@ class Location:
                     entity_type
                     parent_types
                     spec_version
+                    identity_class
                     name
                     description
                     roles
@@ -29,14 +33,10 @@ class Location:
                     created
                     modified
                     objectLabel {
-                        edges {
-                            node {
-                                id
-                                value
-                                color
-                            }
-                        }
-                    }                    
+                        id
+                        value
+                        color
+                    }
                 }
                 ... on Organization {
                     x_opencti_organization_type
@@ -48,28 +48,25 @@ class Location:
                 }
             }
             objectMarking {
-                edges {
-                    node {
-                        id
-                        standard_id
-                        entity_type
-                        definition_type
-                        definition
-                        created
-                        modified
-                        x_opencti_order
-                        x_opencti_color
-                    }
-                }
+                id
+                standard_id
+                entity_type
+                definition_type
+                definition
+                created
+                modified
+                x_opencti_order
+                x_opencti_color
+            }
+            objectOrganization {
+                id
+                standard_id
+                name
             }
             objectLabel {
-                edges {
-                    node {
-                        id
-                        value
-                        color
-                    }
-                }
+                id
+                value
+                color
             }
             externalReferences {
                 edges {
@@ -98,6 +95,129 @@ class Location:
             precision
             x_opencti_aliases
         """
+        self.properties_with_files = """
+            id
+            standard_id
+            entity_type
+            parent_types
+            spec_version
+            created_at
+            updated_at
+            createdBy {
+                ... on Identity {
+                    id
+                    standard_id
+                    entity_type
+                    parent_types
+                    spec_version
+                    identity_class
+                    name
+                    description
+                    roles
+                    contact_information
+                    x_opencti_aliases
+                    created
+                    modified
+                    objectLabel {
+                        id
+                        value
+                        color
+                    }
+                }
+                ... on Organization {
+                    x_opencti_organization_type
+                    x_opencti_reliability
+                }
+                ... on Individual {
+                    x_opencti_firstname
+                    x_opencti_lastname
+                }
+            }
+            objectMarking {
+                id
+                standard_id
+                entity_type
+                definition_type
+                definition
+                created
+                modified
+                x_opencti_order
+                x_opencti_color
+            }
+            objectOrganization {
+                id
+                standard_id
+                name
+            }
+            objectLabel {
+                id
+                value
+                color
+            }
+            externalReferences {
+                edges {
+                    node {
+                        id
+                        standard_id
+                        entity_type
+                        source_name
+                        description
+                        url
+                        hash
+                        external_id
+                        created
+                        modified
+                        importFiles {
+                            edges {
+                                node {
+                                    id
+                                    name
+                                    size
+                                    metaData {
+                                        mimetype
+                                        version
+                                    }
+                                }
+                            }
+                        }
+                    }
+                }
+            }
+            revoked
+            confidence
+            created
+            modified
+            name
+            description
+            latitude
+            longitude
+            precision
+            x_opencti_aliases
+            importFiles {
+                edges {
+                    node {
+                        id
+                        name
+                        size
+                        metaData {
+                            mimetype
+                            version
+                        }
+                    }
+                }
+            }
+        """
+
+    @staticmethod
+    def generate_id(name, x_opencti_location_type, latitude=None, longitude=None):
+        name = name.lower().strip()
+        if x_opencti_location_type == "position":
+            data = {"name": name, "latitude": latitude, "longitude": longitude}
+        else:
+            data = {"name": name, "x_opencti_location_type": x_opencti_location_type}
+        data = canonicalize(data, utf8=False)
+        id = str(uuid.uuid5(uuid.UUID("00abedb4-aa42-466c-9c01-fed23315a9b7"), data))
+        return "location--" + id
 
     """
         List Location objects
@@ -121,20 +241,25 @@ class Location:
         custom_attributes = kwargs.get("customAttributes", None)
         get_all = kwargs.get("getAll", False)
         with_pagination = kwargs.get("withPagination", False)
+        with_files = kwargs.get("withFiles", False)
         if get_all:
             first = 500
 
-        self.opencti.log(
-            "info", "Listing Locations with filters " + json.dumps(filters) + "."
+        self.opencti.app_logger.info(
+            "Listing Locations with filters", {"filters": json.dumps(filters)}
         )
         query = (
             """
-            query Locations($types: [String], $filters: [LocationsFiltering], $search: String, $first: Int, $after: ID, $orderBy: LocationsOrdering, $orderMode: OrderingMode) {
+            query Locations($types: [String], $filters: FilterGroup, $search: String, $first: Int, $after: ID, $orderBy: LocationsOrdering, $orderMode: OrderingMode) {
                 locations(types: $types, filters: $filters, search: $search, first: $first, after: $after, orderBy: $orderBy, orderMode: $orderMode) {
                     edges {
                         node {
                             """
-            + (custom_attributes if custom_attributes is not None else self.properties)
+            + (
+                custom_attributes
+                if custom_attributes is not None
+                else (self.properties_with_files if with_files else self.properties)
+            )
             + """
                         }
                     }
@@ -167,7 +292,7 @@ class Location:
 
     """
         Read a Location object
-        
+
         :param id: the id of the Location
         :param filters: the filters to apply if no id provided
         :return Location object
@@ -177,8 +302,9 @@ class Location:
         id = kwargs.get("id", None)
         filters = kwargs.get("filters", None)
         custom_attributes = kwargs.get("customAttributes", None)
+        with_files = kwargs.get("withFiles", False)
         if id is not None:
-            self.opencti.log("info", "Reading Location {" + id + "}.")
+            self.opencti.app_logger.info("Reading Location", {"id": id})
             query = (
                 """
                 query Location($id: String!) {
@@ -187,7 +313,7 @@ class Location:
                 + (
                     custom_attributes
                     if custom_attributes is not None
-                    else self.properties
+                    else (self.properties_with_files if with_files else self.properties)
                 )
                 + """
                     }
@@ -203,8 +329,8 @@ class Location:
             else:
                 return None
         else:
-            self.opencti.log(
-                "error", "[opencti_location] Missing parameters: id or filters"
+            self.opencti.app_logger.error(
+                "[opencti_location] Missing parameters: id or filters"
             )
             return None
 
@@ -228,17 +354,19 @@ class Location:
         created = kwargs.get("created", None)
         modified = kwargs.get("modified", None)
         name = kwargs.get("name", None)
-        description = kwargs.get("description", "")
+        description = kwargs.get("description", None)
         latitude = kwargs.get("latitude", None)
         longitude = kwargs.get("longitude", None)
         precision = kwargs.get("precision", None)
         x_opencti_aliases = kwargs.get("x_opencti_aliases", None)
+        x_opencti_stix_ids = kwargs.get("x_opencti_stix_ids", None)
+        x_opencti_workflow_id = kwargs.get("x_opencti_workflow_id", None)
         update = kwargs.get("update", False)
 
         if name is not None:
-            self.opencti.log("info", "Creating Location {" + name + "}.")
+            self.opencti.app_logger.info("Creating Location", {"name": name})
             query = """
-                mutation LocationAdd($input: LocationAddInput) {
+                mutation LocationAdd($input: LocationAddInput!) {
                     locationAdd(input: $input) {
                         id
                         standard_id
@@ -268,13 +396,15 @@ class Location:
                         "longitude": longitude,
                         "precision": precision,
                         "x_opencti_aliases": x_opencti_aliases,
+                        "x_opencti_stix_ids": x_opencti_stix_ids,
+                        "x_opencti_workflow_id": x_opencti_workflow_id,
                         "update": update,
                     }
                 },
             )
             return self.opencti.process_multiple_fields(result["data"]["locationAdd"])
         else:
-            self.opencti.log("error", "Missing parameters: name")
+            self.opencti.app_logger.error("Missing parameters: name")
 
     """
         Import an Location object from a STIX2 object
@@ -296,10 +426,12 @@ class Location:
         elif "region" in stix_object:
             name = stix_object["region"]
         else:
-            self.opencti.log("error", "[opencti_location] Missing name")
+            self.opencti.app_logger.error("[opencti_location] Missing name")
             return
         if "x_opencti_location_type" in stix_object:
             type = stix_object["x_opencti_location_type"]
+        elif self.opencti.get_attribute_in_extension("type", stix_object) is not None:
+            type = self.opencti.get_attribute_in_extension("type", stix_object)
         else:
             if "city" in stix_object:
                 type = "City"
@@ -310,45 +442,73 @@ class Location:
             else:
                 type = "Position"
         if stix_object is not None:
+            # Search in extensions
+            if "x_opencti_aliases" not in stix_object:
+                stix_object["x_opencti_aliases"] = (
+                    self.opencti.get_attribute_in_extension("aliases", stix_object)
+                )
+            if "x_opencti_stix_ids" not in stix_object:
+                stix_object["x_opencti_stix_ids"] = (
+                    self.opencti.get_attribute_in_extension("stix_ids", stix_object)
+                )
+            if "x_opencti_workflow_id" not in stix_object:
+                stix_object["x_opencti_workflow_id"] = (
+                    self.opencti.get_attribute_in_extension("workflow_id", stix_object)
+                )
+
             return self.create(
                 type=type,
                 stix_id=stix_object["id"],
-                createdBy=extras["created_by_id"]
-                if "created_by_id" in extras
-                else None,
-                objectMarking=extras["object_marking_ids"]
-                if "object_marking_ids" in extras
-                else [],
-                objectLabel=extras["object_label_ids"]
-                if "object_label_ids" in extras
-                else [],
-                externalReferences=extras["external_references_ids"]
-                if "external_references_ids" in extras
-                else [],
+                createdBy=(
+                    extras["created_by_id"] if "created_by_id" in extras else None
+                ),
+                objectMarking=(
+                    extras["object_marking_ids"]
+                    if "object_marking_ids" in extras
+                    else None
+                ),
+                objectLabel=(
+                    extras["object_label_ids"] if "object_label_ids" in extras else None
+                ),
+                externalReferences=(
+                    extras["external_references_ids"]
+                    if "external_references_ids" in extras
+                    else None
+                ),
                 revoked=stix_object["revoked"] if "revoked" in stix_object else None,
-                confidence=stix_object["confidence"]
-                if "confidence" in stix_object
-                else None,
+                confidence=(
+                    stix_object["confidence"] if "confidence" in stix_object else None
+                ),
                 lang=stix_object["lang"] if "lang" in stix_object else None,
                 created=stix_object["created"] if "created" in stix_object else None,
                 modified=stix_object["modified"] if "modified" in stix_object else None,
                 name=name,
-                description=self.opencti.stix2.convert_markdown(
-                    stix_object["description"]
-                )
-                if "description" in stix_object
-                else "",
+                description=(
+                    self.opencti.stix2.convert_markdown(stix_object["description"])
+                    if "description" in stix_object
+                    else None
+                ),
                 latitude=stix_object["latitude"] if "latitude" in stix_object else None,
-                longitude=stix_object["longitude"]
-                if "longitude" in stix_object
-                else None,
-                precision=stix_object["precision"]
-                if "precision" in stix_object
-                else None,
+                longitude=(
+                    stix_object["longitude"] if "longitude" in stix_object else None
+                ),
+                precision=(
+                    stix_object["precision"] if "precision" in stix_object else None
+                ),
+                x_opencti_stix_ids=(
+                    stix_object["x_opencti_stix_ids"]
+                    if "x_opencti_stix_ids" in stix_object
+                    else None
+                ),
                 x_opencti_aliases=self.opencti.stix2.pick_aliases(stix_object),
+                x_opencti_workflow_id=(
+                    stix_object["x_opencti_workflow_id"]
+                    if "x_opencti_workflow_id" in stix_object
+                    else None
+                ),
                 update=update,
             )
         else:
-            self.opencti.log(
-                "error", "[opencti_location] Missing parameters: stixObject"
+            self.opencti.app_logger.error(
+                "[opencti_location] Missing parameters: stixObject"
             )
