@@ -5,8 +5,6 @@ import uuid
 
 from stix2.canonicalization.Canonicalize import canonicalize
 
-from pycti.entities import LOGGER
-
 
 class StixCoreRelationship:
     def __init__(self, opencti):
@@ -44,13 +42,9 @@ class StixCoreRelationship:
                     created
                     modified
                     objectLabel {
-                        edges {
-                            node {
-                                id
-                                value
-                                color
-                            }
-                        }
+                        id
+                        value
+                        color
                     }
                 }
                 ... on Organization {
@@ -63,28 +57,25 @@ class StixCoreRelationship:
                 }
             }
             objectMarking {
-                edges {
-                    node {
-                        id
-                        standard_id
-                        entity_type
-                        definition_type
-                        definition
-                        created
-                        modified
-                        x_opencti_order
-                        x_opencti_color
-                    }
-                }
+                id
+                standard_id
+                entity_type
+                definition_type
+                definition
+                created
+                modified
+                x_opencti_order
+                x_opencti_color
+            }
+            objectOrganization {
+                id
+                standard_id
+                name
             }
             objectLabel {
-                edges {
-                    node {
-                        id
-                        value
-                        color
-                    }
-                }
+                id
+                value
+                color
             }
             externalReferences {
                 edges {
@@ -386,7 +377,7 @@ class StixCoreRelationship:
     """
 
     def list(self, **kwargs):
-        element_id = kwargs.get("elementId", None)
+        from_or_to_id = kwargs.get("fromOrToId", None)
         element_with_target_types = kwargs.get("elementWithTargetTypes", None)
         from_id = kwargs.get("fromId", None)
         from_types = kwargs.get("fromTypes", None)
@@ -397,7 +388,7 @@ class StixCoreRelationship:
         start_time_stop = kwargs.get("startTimeStop", None)
         stop_time_start = kwargs.get("stopTimeStart", None)
         stop_time_stop = kwargs.get("stopTimeStop", None)
-        filters = kwargs.get("filters", [])
+        filters = kwargs.get("filters", None)
         first = kwargs.get("first", 100)
         after = kwargs.get("after", None)
         order_by = kwargs.get("orderBy", None)
@@ -405,19 +396,27 @@ class StixCoreRelationship:
         custom_attributes = kwargs.get("customAttributes", None)
         get_all = kwargs.get("getAll", False)
         with_pagination = kwargs.get("withPagination", False)
+        search = kwargs.get("search", None)
         if get_all:
             first = 100
 
-        LOGGER.info(
-            "Listing stix_core_relationships with {type: %s, element_id: %s, from_id: %s, "
-            "to_id: %s, element_with_target_types: %s, from_types: %s, to_types: %s}",
-            *(relationship_type, element_id, from_id),
-            *(to_id, element_with_target_types, from_types, to_types),
+        self.opencti.app_logger.info(
+            "Listing stix_core_relationships",
+            {
+                "relationship_type": relationship_type,
+                "from_or_to_id": from_or_to_id,
+                "from_id": from_id,
+                "to_id": to_id,
+                "element_with_target_types": element_with_target_types,
+                "from_types": from_types,
+                "to_types": to_types,
+                "search": search,
+            },
         )
         query = (
             """
-                query StixCoreRelationships($elementId: [String], $elementWithTargetTypes: [String], $fromId: [String], $fromTypes: [String], $toId: [String], $toTypes: [String], $relationship_type: [String], $startTimeStart: DateTime, $startTimeStop: DateTime, $stopTimeStart: DateTime, $stopTimeStop: DateTime, $filters: [StixCoreRelationshipsFiltering], $first: Int, $after: ID, $orderBy: StixCoreRelationshipsOrdering, $orderMode: OrderingMode) {
-                    stixCoreRelationships(elementId: $elementId, elementWithTargetTypes: $elementWithTargetTypes, fromId: $fromId, fromTypes: $fromTypes, toId: $toId, toTypes: $toTypes, relationship_type: $relationship_type, startTimeStart: $startTimeStart, startTimeStop: $startTimeStop, stopTimeStart: $stopTimeStart, stopTimeStop: $stopTimeStop, filters: $filters, first: $first, after: $after, orderBy: $orderBy, orderMode: $orderMode) {
+                query StixCoreRelationships($fromOrToId: [String], $elementWithTargetTypes: [String], $fromId: [String], $fromTypes: [String], $toId: [String], $toTypes: [String], $relationship_type: [String], $startTimeStart: DateTime, $startTimeStop: DateTime, $stopTimeStart: DateTime, $stopTimeStop: DateTime, $filters: FilterGroup, $first: Int, $after: ID, $orderBy: StixCoreRelationshipsOrdering, $orderMode: OrderingMode, $search: String) {
+                    stixCoreRelationships(fromOrToId: $fromOrToId, elementWithTargetTypes: $elementWithTargetTypes, fromId: $fromId, fromTypes: $fromTypes, toId: $toId, toTypes: $toTypes, relationship_type: $relationship_type, startTimeStart: $startTimeStart, startTimeStop: $startTimeStop, stopTimeStart: $stopTimeStart, stopTimeStop: $stopTimeStop, filters: $filters, first: $first, after: $after, orderBy: $orderBy, orderMode: $orderMode, search: $search) {
                         edges {
                             node {
                                 """
@@ -439,7 +438,7 @@ class StixCoreRelationship:
         result = self.opencti.query(
             query,
             {
-                "elementId": element_id,
+                "fromOrToId": from_or_to_id,
                 "elementWithTargetTypes": element_with_target_types,
                 "fromId": from_id,
                 "fromTypes": from_types,
@@ -455,6 +454,7 @@ class StixCoreRelationship:
                 "after": after,
                 "orderBy": order_by,
                 "orderMode": order_mode,
+                "search": search,
             },
         )
         if get_all:
@@ -465,11 +465,13 @@ class StixCoreRelationship:
             final_data = final_data + data
             while result["data"]["stixCoreRelationships"]["pageInfo"]["hasNextPage"]:
                 after = result["data"]["stixCoreRelationships"]["pageInfo"]["endCursor"]
-                LOGGER.info("Listing StixCoreRelationships after " + after)
+                self.opencti.app_logger.info(
+                    "Listing StixCoreRelationships", {"after": after}
+                )
                 result = self.opencti.query(
                     query,
                     {
-                        "elementId": element_id,
+                        "fromOrToId": from_or_to_id,
                         "elementWithTargetTypes": element_with_target_types,
                         "fromId": from_id,
                         "fromTypes": from_types,
@@ -501,7 +503,7 @@ class StixCoreRelationship:
         Read a stix_core_relationship object
 
         :param id: the id of the stix_core_relationship
-        :param elementId: the id of the entity of the relation
+        :param fromOrToId: the id of the entity of the relation
         :param fromId: the id of the source entity of the relation
         :param toId: the id of the target entity of the relation
         :param relationship_type: the relation type
@@ -514,7 +516,7 @@ class StixCoreRelationship:
 
     def read(self, **kwargs):
         id = kwargs.get("id", None)
-        element_id = kwargs.get("elementId", None)
+        from_or_to_id = kwargs.get("fromOrToId", None)
         from_id = kwargs.get("fromId", None)
         to_id = kwargs.get("toId", None)
         relationship_type = kwargs.get("relationship_type", None)
@@ -522,9 +524,10 @@ class StixCoreRelationship:
         start_time_stop = kwargs.get("startTimeStop", None)
         stop_time_start = kwargs.get("stopTimeStart", None)
         stop_time_stop = kwargs.get("stopTimeStop", None)
+        filters = kwargs.get("filters", None)
         custom_attributes = kwargs.get("customAttributes", None)
         if id is not None:
-            LOGGER.info("Reading stix_core_relationship {%s}.", id)
+            self.opencti.app_logger.info("Reading stix_core_relationship", {"id": id})
             query = (
                 """
                     query StixCoreRelationship($id: String!) {
@@ -544,9 +547,15 @@ class StixCoreRelationship:
             return self.opencti.process_multiple_fields(
                 result["data"]["stixCoreRelationship"]
             )
+        elif filters is not None:
+            result = self.list(filters=filters, customAttributes=custom_attributes)
+            if len(result) > 0:
+                return result[0]
+            else:
+                return None
         elif from_id is not None and to_id is not None:
             result = self.list(
-                elementId=element_id,
+                fromOrToId=from_or_to_id,
                 fromId=from_id,
                 toId=to_id,
                 relationship_type=relationship_type,
@@ -560,7 +569,7 @@ class StixCoreRelationship:
             else:
                 return None
         else:
-            LOGGER.error("Missing parameters: id or from_id and to_id")
+            self.opencti.app_logger.error("Missing parameters: id or from_id and to_id")
             return None
 
     """
@@ -589,13 +598,16 @@ class StixCoreRelationship:
         external_references = kwargs.get("externalReferences", None)
         kill_chain_phases = kwargs.get("killChainPhases", None)
         granted_refs = kwargs.get("objectOrganization", None)
+        x_opencti_workflow_id = kwargs.get("x_opencti_workflow_id", None)
         update = kwargs.get("update", False)
 
-        LOGGER.info(
-            "Creating stix_core_relationship '%s' {%s, %s}.",
-            relationship_type,
-            from_id,
-            to_id,
+        self.opencti.app_logger.info(
+            "Creating stix_core_relationship",
+            {
+                "relationship_type": relationship_type,
+                "from_id": from_id,
+                "to_id": to_id,
+            },
         )
         query = """
                 mutation StixCoreRelationshipAdd($input: StixCoreRelationshipAddInput!) {
@@ -629,6 +641,7 @@ class StixCoreRelationship:
                     "objectOrganization": granted_refs,
                     "externalReferences": external_references,
                     "killChainPhases": kill_chain_phases,
+                    "x_opencti_workflow_id": x_opencti_workflow_id,
                     "update": update,
                 }
             },
@@ -649,7 +662,7 @@ class StixCoreRelationship:
         id = kwargs.get("id", None)
         input = kwargs.get("input", None)
         if id is not None and input is not None:
-            LOGGER.info("Updating stix_core_relationship {%s}.", id)
+            self.opencti.app_logger.info("Updating stix_core_relationship", {"id": id})
             query = """
                     mutation StixCoreRelationshipEdit($id: ID!, $input: [EditInput]!) {
                         stixCoreRelationshipEdit(id: $id) {
@@ -672,7 +685,7 @@ class StixCoreRelationship:
                 result["data"]["stixCoreRelationshipEdit"]["fieldPatch"]
             )
         else:
-            LOGGER.error(
+            self.opencti.app_logger.error(
                 "[opencti_stix_core_relationship] Missing parameters: id and key and value",
             )
             return None
@@ -687,7 +700,7 @@ class StixCoreRelationship:
     def delete(self, **kwargs):
         id = kwargs.get("id", None)
         if id is not None:
-            LOGGER.info("Deleting stix_core_relationship {%s}.", id)
+            self.opencti.app_logger.info("Deleting stix_core_relationship", {"id": id})
             query = """
                 mutation StixCoreRelationshipEdit($id: ID!) {
                     stixCoreRelationshipEdit(id: $id) {
@@ -697,7 +710,9 @@ class StixCoreRelationship:
             """
             self.opencti.query(query, {"id": id})
         else:
-            LOGGER.error("[opencti_stix_core_relationship] Missing parameters: id")
+            self.opencti.app_logger.error(
+                "[opencti_stix_core_relationship] Missing parameters: id"
+            )
             return None
 
     """
@@ -715,36 +730,34 @@ class StixCoreRelationship:
             custom_attributes = """
                 id
                 objectMarking {
-                    edges {
-                        node {
-                            id
-                            standard_id
-                            entity_type
-                            definition_type
-                            definition
-                            x_opencti_order
-                            x_opencti_color
-                            created
-                            modified
-                        }
-                    }
+                    id
+                    standard_id
+                    entity_type
+                    definition_type
+                    definition
+                    x_opencti_order
+                    x_opencti_color
+                    created
+                    modified
                 }
             """
             stix_core_relationship = self.read(
                 id=id, customAttributes=custom_attributes
             )
             if stix_core_relationship is None:
-                LOGGER.error("Cannot add Marking-Definition, entity not found")
+                self.opencti.app_logger.error(
+                    "Cannot add Marking-Definition, entity not found"
+                )
                 return False
             if marking_definition_id in stix_core_relationship["objectMarkingIds"]:
                 return True
             else:
-                LOGGER.info(
-                    "Adding Marking-Definition {%s} to Stix-Domain-Object {%s}",
-                    *(marking_definition_id, id),
+                self.opencti.app_logger.info(
+                    "Adding Marking-Definition to Stix-Domain-Object",
+                    {"id": id, "marking_definition_id": marking_definition_id},
                 )
                 query = """
-                   mutation StixCoreRelationshipAddRelation($id: ID!, $input: StixMetaRelationshipAddInput) {
+                   mutation StixCoreRelationshipAddRelation($id: ID!, $input: StixRefRelationshipAddInput!) {
                        stixCoreRelationshipEdit(id: $id) {
                             relationAdd(input: $input) {
                                 id
@@ -764,7 +777,9 @@ class StixCoreRelationship:
                 )
                 return True
         else:
-            LOGGER.error("Missing parameters: id and marking_definition_id")
+            self.opencti.app_logger.error(
+                "Missing parameters: id and marking_definition_id"
+            )
             return False
 
     """
@@ -779,9 +794,9 @@ class StixCoreRelationship:
         id = kwargs.get("id", None)
         marking_definition_id = kwargs.get("marking_definition_id", None)
         if id is not None and marking_definition_id is not None:
-            LOGGER.info(
-                "Removing Marking-Definition {%s} from stix_core_relationship {%s}",
-                *(marking_definition_id, id),
+            self.opencti.app_logger.info(
+                "Removing Marking-Definition from stix_core_relationship",
+                {"id": id, "marking_definition_id": marking_definition_id},
             )
             query = """
                mutation StixCoreRelationshipRemoveRelation($id: ID!, $toId: StixRef!, $relationship_type: String!) {
@@ -802,7 +817,7 @@ class StixCoreRelationship:
             )
             return True
         else:
-            LOGGER.error("Missing parameters: id and label_id")
+            self.opencti.app_logger.error("Missing parameters: id and label_id")
             return False
 
     """
@@ -819,7 +834,11 @@ class StixCoreRelationship:
         label_name = kwargs.get("label_name", None)
         if label_name is not None:
             label = self.opencti.label.read(
-                filters=[{"key": "value", "values": [label_name]}]
+                filters={
+                    "mode": "and",
+                    "filters": [{"key": "value", "values": [label_name]}],
+                    "filterGroups": [],
+                }
             )
             if label:
                 label_id = label["id"]
@@ -827,11 +846,12 @@ class StixCoreRelationship:
                 label = self.opencti.label.create(value=label_name)
                 label_id = label["id"]
         if id is not None and label_id is not None:
-            LOGGER.info(
-                "Adding label {%s} to stix-core-relationship {%s}", label_id, id
+            self.opencti.app_logger.info(
+                "Adding label to stix-core-relationship",
+                {"label_id": label_id, "id": id},
             )
             query = """
-               mutation StixCoreRelationshipAddRelation($id: ID!, $input: StixMetaRelationshipAddInput) {
+               mutation StixCoreRelationshipAddRelation($id: ID!, $input: StixRefRelationshipAddInput!) {
                    stixCoreRelationshipEdit(id: $id) {
                         relationAdd(input: $input) {
                             id
@@ -851,7 +871,7 @@ class StixCoreRelationship:
             )
             return True
         else:
-            LOGGER.error("Missing parameters: id and label_id")
+            self.opencti.app_logger.error("Missing parameters: id and label_id")
             return False
 
     """
@@ -866,12 +886,12 @@ class StixCoreRelationship:
         id = kwargs.get("id", None)
         external_reference_id = kwargs.get("external_reference_id", None)
         if id is not None and external_reference_id is not None:
-            LOGGER.info(
-                "Adding External-Reference {%s} to stix-core-relationship {%s}",
-                *(external_reference_id, id),
+            self.opencti.app_logger.info(
+                "Adding External-Reference to stix-core-relationship",
+                {"external_reference_id": external_reference_id, "id": id},
             )
             query = """
-               mutation StixCoreRelationshipEditRelationAdd($id: ID!, $input: StixMetaRelationshipAddInput) {
+               mutation StixCoreRelationshipEditRelationAdd($id: ID!, $input: StixRefRelationshipAddInput!) {
                    stixCoreRelationshipEdit(id: $id) {
                         relationAdd(input: $input) {
                             id
@@ -891,7 +911,9 @@ class StixCoreRelationship:
             )
             return True
         else:
-            LOGGER.error("Missing parameters: id and external_reference_id")
+            self.opencti.app_logger.error(
+                "Missing parameters: id and external_reference_id"
+            )
             return False
 
     """
@@ -906,9 +928,9 @@ class StixCoreRelationship:
         id = kwargs.get("id", None)
         external_reference_id = kwargs.get("external_reference_id", None)
         if id is not None and external_reference_id is not None:
-            LOGGER.info(
-                "Removing External-Reference {%s} from stix_core_relationship {%s}",
-                *(external_reference_id, id),
+            self.opencti.app_logger.info(
+                "Removing External-Reference from stix_core_relationship",
+                {"external_reference_id": external_reference_id, "id": id},
             )
             query = """
                mutation StixCoreRelationshipRemoveRelation($id: ID!, $toId: StixRef!, $relationship_type: String!) {
@@ -929,7 +951,7 @@ class StixCoreRelationship:
             )
             return True
         else:
-            LOGGER.error("Missing parameters: id and label_id")
+            self.opencti.app_logger.error("Missing parameters: id and label_id")
             return False
 
     """
@@ -944,12 +966,12 @@ class StixCoreRelationship:
         id = kwargs.get("id", None)
         kill_chain_phase_id = kwargs.get("kill_chain_phase_id", None)
         if id is not None and kill_chain_phase_id is not None:
-            LOGGER.info(
-                "Adding Kill-Chain-Phase {%s} to stix-core-relationship {%s}",
-                *(kill_chain_phase_id, id),
+            self.opencti.app_logger.info(
+                "Adding Kill-Chain-Phase to stix-core-relationship",
+                {"kill_chain_phase_id": kill_chain_phase_id, "id": id},
             )
             query = """
-               mutation StixCoreRelationshipAddRelation($id: ID!, $input: StixMetaRelationshipAddInput) {
+               mutation StixCoreRelationshipAddRelation($id: ID!, $input: StixRefRelationshipAddInput!) {
                    stixCoreRelationshipEdit(id: $id) {
                         relationAdd(input: $input) {
                             id
@@ -969,7 +991,7 @@ class StixCoreRelationship:
             )
             return True
         else:
-            LOGGER.error(
+            self.opencti.app_logger.error(
                 "[opencti_stix_core_relationship] Missing parameters: id and kill_chain_phase_id",
             )
             return False
@@ -986,9 +1008,9 @@ class StixCoreRelationship:
         id = kwargs.get("id", None)
         kill_chain_phase_id = kwargs.get("kill_chain_phase_id", None)
         if id is not None and kill_chain_phase_id is not None:
-            LOGGER.info(
-                "Removing Kill-Chain-Phase {%s} from stix_core_relationship {%s}",
-                *(kill_chain_phase_id, id),
+            self.opencti.app_logger.info(
+                "Removing Kill-Chain-Phase from stix_core_relationship",
+                {"kill_chain_phase_id": kill_chain_phase_id, "id": id},
             )
             query = """
                mutation StixCoreRelationshipRemoveRelation($id: ID!, $toId: StixRef!, $relationship_type: String!) {
@@ -1009,7 +1031,7 @@ class StixCoreRelationship:
             )
             return True
         else:
-            LOGGER.error(
+            self.opencti.app_loggererror(
                 "[stix_core_relationship] Missing parameters: id and kill_chain_phase_id"
             )
             return False
@@ -1026,9 +1048,9 @@ class StixCoreRelationship:
         id = kwargs.get("id", None)
         identity_id = kwargs.get("identity_id", None)
         if id is not None:
-            LOGGER.info(
-                "Updating author of stix_core_relationship {%s} with Identity {%s}",
-                *(id, identity_id),
+            self.opencti.app_logger.info(
+                "Updating author of stix_core_relationship with Identity",
+                {"id": id, "identity_id": identity_id},
             )
             custom_attributes = """
                 id
@@ -1076,7 +1098,7 @@ class StixCoreRelationship:
             if identity_id is not None:
                 # Add the new relation
                 query = """
-                    mutation StixCoreRelationshipEdit($id: ID!, $input: StixMetaRelationshipAddInput) {
+                    mutation StixCoreRelationshipEdit($id: ID!, $input: StixRefRelationshipAddInput!) {
                         stixCoreRelationshipEdit(id: $id) {
                             relationAdd(input: $input) {
                                 id
@@ -1093,7 +1115,7 @@ class StixCoreRelationship:
                 }
                 self.opencti.query(query, variables)
         else:
-            LOGGER.error("Missing parameters: id")
+            self.opencti.app_logger.error("Missing parameters: id")
             return False
 
     """
@@ -1110,9 +1132,17 @@ class StixCoreRelationship:
         default_date = kwargs.get("defaultDate", False)
         if stix_relation is not None:
             # Search in extensions
-            if "granted_refs" not in stix_relation:
-                stix_relation["granted_refs"] = self.opencti.get_attribute_in_extension(
-                    "granted_refs", stix_relation
+            if "x_opencti_granted_refs" not in stix_relation:
+                stix_relation["x_opencti_granted_refs"] = (
+                    self.opencti.get_attribute_in_extension(
+                        "granted_refs", stix_relation
+                    )
+                )
+            if "x_opencti_workflow_id" not in stix_relation:
+                stix_relation["x_opencti_workflow_id"] = (
+                    self.opencti.get_attribute_in_extension(
+                        "workflow_id", stix_relation
+                    )
                 )
 
             source_ref = stix_relation["source_ref"]
@@ -1122,49 +1152,70 @@ class StixCoreRelationship:
                 toId=target_ref,
                 stix_id=stix_relation["id"],
                 relationship_type=stix_relation["relationship_type"],
-                description=self.opencti.stix2.convert_markdown(
-                    stix_relation["description"]
-                )
-                if "description" in stix_relation
-                else "",
-                start_time=stix_relation["start_time"]
-                if "start_time" in stix_relation
-                else default_date,
-                stop_time=stix_relation["stop_time"]
-                if "stop_time" in stix_relation
-                else default_date,
-                revoked=stix_relation["revoked"]
-                if "revoked" in stix_relation
-                else None,
-                confidence=stix_relation["confidence"]
-                if "confidence" in stix_relation
-                else None,
+                description=(
+                    self.opencti.stix2.convert_markdown(stix_relation["description"])
+                    if "description" in stix_relation
+                    else None
+                ),
+                start_time=(
+                    stix_relation["start_time"]
+                    if "start_time" in stix_relation
+                    else default_date
+                ),
+                stop_time=(
+                    stix_relation["stop_time"]
+                    if "stop_time" in stix_relation
+                    else default_date
+                ),
+                revoked=(
+                    stix_relation["revoked"] if "revoked" in stix_relation else None
+                ),
+                confidence=(
+                    stix_relation["confidence"]
+                    if "confidence" in stix_relation
+                    else None
+                ),
                 lang=stix_relation["lang"] if "lang" in stix_relation else None,
-                created=stix_relation["created"]
-                if "created" in stix_relation
-                else None,
-                modified=stix_relation["modified"]
-                if "modified" in stix_relation
-                else None,
-                createdBy=extras["created_by_id"]
-                if "created_by_id" in extras
-                else None,
-                objectMarking=extras["object_marking_ids"]
-                if "object_marking_ids" in extras
-                else None,
-                objectLabel=extras["object_label_ids"]
-                if "object_label_ids" in extras
-                else [],
-                externalReferences=extras["external_references_ids"]
-                if "external_references_ids" in extras
-                else [],
-                killChainPhases=extras["kill_chain_phases_ids"]
-                if "kill_chain_phases_ids" in extras
-                else None,
-                objectOrganization=stix_relation["granted_refs"]
-                if "granted_refs" in stix_relation
-                else None,
+                created=(
+                    stix_relation["created"] if "created" in stix_relation else None
+                ),
+                modified=(
+                    stix_relation["modified"] if "modified" in stix_relation else None
+                ),
+                createdBy=(
+                    extras["created_by_id"] if "created_by_id" in extras else None
+                ),
+                objectMarking=(
+                    extras["object_marking_ids"]
+                    if "object_marking_ids" in extras
+                    else None
+                ),
+                objectLabel=(
+                    extras["object_label_ids"] if "object_label_ids" in extras else None
+                ),
+                externalReferences=(
+                    extras["external_references_ids"]
+                    if "external_references_ids" in extras
+                    else None
+                ),
+                killChainPhases=(
+                    extras["kill_chain_phases_ids"]
+                    if "kill_chain_phases_ids" in extras
+                    else None
+                ),
+                objectOrganization=(
+                    stix_relation["x_opencti_granted_refs"]
+                    if "x_opencti_granted_refs" in stix_relation
+                    else None
+                ),
+                x_opencti_workflow_id=(
+                    stix_relation["x_opencti_workflow_id"]
+                    if "x_opencti_workflow_id" in stix_relation
+                    else None
+                ),
                 update=update,
             )
         else:
-            LOGGER.error("[opencti_attack_pattern] Missing parameters: stixObject")
+            self.opencti.app_logger.error(
+                "[opencti_stix_core_relationship] Missing parameters: stixObject"
+            )
